@@ -41,19 +41,7 @@ class TOCEntry(Container):
     "Create the TOC entry for a container, consisting of a single link."
     if container.partkey.header:
       return self.header(container)
-    labels = container.searchall(Label)
-    if len(labels) == 0 or Options.toc:
-      url = Options.toctarget + '#' + container.partkey.partkey
-      link = Link().complete(container.partkey.tocentry, url=url)
-    else:
-      label = labels[0]
-      link = Link().complete(container.partkey.tocentry)
-      link.destination = label
-    self.contents = [link]
-    titlecontents = self.gettitlecontents(container)
-    if container.partkey.tocsuffix and titlecontents:
-      link.contents.append(Constant(container.partkey.tocsuffix))
-      link.contents += titlecontents
+    self.contents = [self.createlink(container)]
     self.output = TaggedOutput().settag('div class="toc"', True)
     self.partkey = container.partkey
     return self
@@ -64,11 +52,35 @@ class TOCEntry(Container):
     self.output = EmptyOutput()
     return self
 
+  def createlink(self, container):
+    "Create the link that will make the whole TOC entry."
+    labels = container.searchall(Label)
+    link = Link()
+    if len(labels) == 0 or Options.toc:
+      link.url = Options.toctarget + '#' + container.partkey.partkey
+    else:
+      label = labels[0]
+      link.destination = label
+    if container.partkey.tocentry:
+      link.complete(container.partkey.tocentry)
+    titlecontents = self.gettitlecontents(container)
+    if titlecontents:
+      if Options.notoclabels:
+        separator = u'. '
+      else:
+        separator = u': '
+      if container.partkey.tocentry:
+        link.contents.append(Constant(separator))
+      link.contents += titlecontents
+    return link
+
   def gettitlecontents(self, container):
     "Get the title of the container."
+    if not container.partkey.showtitle:
+      return None
     shorttitles = container.searchall(ShortTitle)
     if len(shorttitles) > 0:
-      contents = [Constant(u' ')]
+      contents = []
       for shorttitle in shorttitles:
         contents += shorttitle.contents
       return contents
@@ -80,6 +92,8 @@ class TOCEntry(Container):
 
   def __unicode__(self):
     "Return a printable representation."
+    if not self.partkey.tocentry:
+      return 'Unnamed TOC entry'
     return 'TOC entry: ' + self.partkey.tocentry
 
 class Indenter(object):
